@@ -1,8 +1,10 @@
 #include "dnswriter.hh"
 #include "misc.hh"
 #include "dnsparser.hh"
+#include <boost/tokenizer.hpp>
+#include <boost/algorithm/string.hpp>
 
-DNSPacketWriter::DNSPacketWriter(vector<uint8_t>& content, const string& qname, uint16_t  qtype, uint16_t qclass)
+DNSPacketWriter::DNSPacketWriter(vector<uint8_t>& content, const string& qname, uint16_t  qtype, uint16_t qclass, uint8_t opcode)
   : d_pos(0), d_content(content), d_qname(qname), d_qtype(qtype), d_qclass(qclass)
 {
   d_content.clear();
@@ -11,46 +13,46 @@ DNSPacketWriter::DNSPacketWriter(vector<uint8_t>& content, const string& qname,
   memset(&dnsheader, 0, sizeof(dnsheader));
   dnsheader.id=0;
   dnsheader.qdcount=htons(1);
+  dnsheader.opcode=opcode;
   
   const uint8_t* ptr=(const uint8_t*)&dnsheader;
   uint32_t len=d_content.size();
   d_content.resize(len + sizeof(dnsheader));
   uint8_t* dptr=(&*d_content.begin()) + len;
-
-
-  memcpy(dptr, ptr, sizeof(dnsheader));
-
-  //  d_content.insert(d_content.end(), ptr, ptr + sizeof(dnsheader));    
   
+  memcpy(dptr, ptr, sizeof(dnsheader));
+  d_stuff=0;
+
   xfrLabel(qname, false);
   
   len=d_content.size();
-  d_content.resize(len + d_record.size());
+  d_content.resize(len + d_record.size() + 4);
+
   ptr=&*d_record.begin();
   dptr=(&*d_content.begin()) + len;
   
   memcpy(dptr, ptr, d_record.size());
-  //  d_content.insert(d_content.end(), d_record.begin(), d_record.end());
 
+  len+=d_record.size();
   d_record.clear();
 
   qtype=htons(qtype);
-  ptr=(const uint8_t*)&qtype;
-  d_content.insert(d_content.end(), ptr, ptr+2);
-  
   qclass=htons(qclass);
-  ptr=(const uint8_t*)&qclass;
-  d_content.insert(d_content.end(), ptr, ptr+2);
+
+  vector<uint8_t>::iterator i=d_content.begin()+len; // this works around a gcc 3.4 bug
+  memcpy(&*i, &qtype, 2);
+  i+=2;
+  memcpy(&*i, &qclass, 2);
 
   d_stuff=0xffff;
+  d_labelmap.reserve(16);
 }
 
-DNSPacketWriter::dnsheader* DNSPacketWriter::getHeader()
+dnsheader* DNSPacketWriter::getHeader()
 {
   return (dnsheader*)&*d_content.begin();
 }
 
-
 void DNSPacketWriter::startRecord(const string& name, uint16_t qtype, uint32_t ttl, uint16_t qclass, Place place)
 {
   if(!d_record.empty()) 
@@ -65,10 +67,16 @@ void DNSPacketWriter::startRecord(const string& name, uint16_t qtype, uint32_t t
   d_stuff = 0; 
   d_rollbackmarker=d_content.size();
 
-  xfrLabel(d_recordqname, true);
-  d_content.insert(d_content.end(), d_record.begin(), d_record.end());
-  d_record.clear();
-
+  if(d_qname == d_recordqname) {  // don't do the whole label compression thing if we *know* we can get away with "see question"
+    static char marker[2]={0xc0, 0x0c};
+    d_content.insert(d_content.end(), &marker[0], &marker[2]);
+  }
+  else {
+    xfrLabel(d_recordqname, true);
+    d_content.insert(d_content.end(), d_record.begin(), d_record.end());
+    d_record.clear();
+  }
+      
   d_stuff = sizeof(dnsrecordheader); // this is needed to get compressed label offsets right, the dnsrecordheader will be interspersed
   d_sor=d_content.size() + d_stuff; // start of real record 
 }
@@ -76,12 +84,8 @@ void DNSPacketWriter::startRecord(const string& name, uint16_t qtype, uint32_t t
 void DNSPacketWriter::addOpt(int udpsize, int extRCode, int Z)
 {
   uint32_t ttl=0;
-  struct Stuff {
-    uint8_t extRCode, version;
-    uint16_t Z;
-  } __attribute__((packed));
 
-  Stuff stuff;
+  EDNS0Record stuff;
 
   stuff.extRCode=extRCode;
   stuff.version=0;
@@ -103,7 +107,7 @@ void DNSPacketWriter::xfr32BitInt(uint32_t val)
 
 void DNSPacketWriter::xfr16BitInt(uint16_t val)
 {
-  int rval=htons(val);
+  uint16_t rval=htons(val);
   uint8_t* ptr=reinterpret_cast<uint8_t*>(&rval);
   d_record.insert(d_record.end(), ptr, ptr+2);
 }
@@ -113,47 +117,111 @@ void DNSPacketWriter::xfr8BitInt(uint8_t val)
   d_record.push_back(val);
 }
 
-void DNSPacketWriter::xfrText(const string& text)
+void DNSPacketWriter::xfrText(const string& text, bool)
 {
-  d_record.push_back(text.length());
-  const uint8_t* ptr=(uint8_t*)(text.c_str());
-  d_record.insert(d_record.end(), ptr, ptr+text.size());
+  escaped_list_separator<char> sep('\\', ' ' , '"');
+  tokenizer<escaped_list_separator<char> > tok(text, sep);
+
+  tokenizer<escaped_list_separator<char> >::iterator beg=tok.begin();
+
+  if(beg==tok.end()) {
+    d_record.push_back(0);
+  }
+  else 
+    for(; beg!=tok.end(); ++beg){
+      d_record.push_back(beg->length());
+      const uint8_t* ptr=(uint8_t*)(beg->c_str());
+      d_record.insert(d_record.end(), ptr, ptr+beg->length());
+    }
 }
 
+DNSPacketWriter::lmap_t::iterator find(DNSPacketWriter::lmap_t& lmap, const string& label)
+{
+  DNSPacketWriter::lmap_t::iterator ret;
+  for(ret=lmap.begin(); ret != lmap.end(); ++ret)
+    if(ret->first == label)
+      break;
+  return ret;
+}
 
+typedef vector<pair<string::size_type, string::size_type> > parts_t;
+
+bool labeltokUnescape(parts_t& parts, const string& label)
+{
+  string::size_type epos = label.size(), lpos(0), pos;
+  bool unescapedSomething = false;
+  const char* ptr=label.c_str();
+
+  parts.clear();
+
+  for(pos = 0 ; pos < epos; ++pos) {
+    if(ptr[pos]=='\\') {
+      pos++;
+      unescapedSomething = true;
+      continue;
+    }
+    if(ptr[pos]=='.') {
+      parts.push_back(make_pair(lpos, pos));
+      lpos=pos+1;
+    }
+  }
+  
+  if(lpos < pos)
+    parts.push_back(make_pair(lpos, pos));
+  return unescapedSomething;
+}
+
+// this is the absolute hottest function in the pdns recursor 
 void DNSPacketWriter::xfrLabel(const string& label, bool compress)
 {
-  typedef vector<string> parts_t;
   parts_t parts;
-  stringtok(parts, label, ".");
-  
-  string enc;
-  unsigned int pos=d_content.size() + d_record.size() + d_stuff; // d_stuff is amount of stuff that is yet to be written out - the dnsrecordheader for example
-  string chopped(label);
+
+  if(label.size()==1 && label[0]=='.') { // otherwise we encode '..'
+    d_record.push_back(0);
+    return;
+  }
+
+  bool unescaped=labeltokUnescape(parts, label); 
   
+  // d_stuff is amount of stuff that is yet to be written out - the dnsrecordheader for example
+  unsigned int pos=d_content.size() + d_record.size() + d_stuff; 
+  string chopped;
   for(parts_t::const_iterator i=parts.begin(); i!=parts.end(); ++i) {
-    map<string, uint16_t>::iterator li;
-    if(compress && (li=d_labelmap.find(chopped))!=d_labelmap.end()) {   // see if we've written out this domain before
+    chopped.assign(label.c_str() + i->first);
+    lmap_t::iterator li=d_labelmap.end();
+    // see if we've written out this domain before
+    if(compress && (li=find(d_labelmap, chopped))!=d_labelmap.end()) {   
       uint16_t offset=li->second;
       offset|=0xc000;
-      enc.append(1, (char)(offset >> 8));
-      enc.append(1, (char)(offset & 0xff));
+      d_record.push_back((char)(offset >> 8));
+      d_record.push_back((char)(offset & 0xff));
       goto out;                                 // skip trailing 0 in case of compression
     }
-    else if(compress || d_labelmap.count(chopped)) { // if 'compress' is true, li will be equal to d_labelmap.end()
-      d_labelmap[chopped]=pos;                       //  if untrue, we need to count 
+
+    if(li==d_labelmap.end() && pos< 16384)
+      d_labelmap.push_back(make_pair(chopped, pos));                       //  if untrue, we need to count - also, don't store offsets > 16384, won't work
+
+    if(unescaped) {
+      string part(label.c_str() + i -> first, i->second - i->first);
+      replace_all(part, "\\.", ".");
+      d_record.push_back(part.size());
+      unsigned int len=d_record.size();
+      d_record.resize(len + part.size());
+
+      memcpy(((&*d_record.begin()) + len), part.c_str(), part.size());
+      pos+=(part.size())+1;			 
+    }
+    else {
+      d_record.push_back((char)(i->second - i->first));
+      unsigned int len=d_record.size();
+      d_record.resize(len + i->second - i->first);
+      memcpy(((&*d_record.begin()) + len), label.c_str() + i-> first, i->second - i->first);
+      pos+=(i->second - i->first)+1;
     }
-    enc.append(1, (char)i->length());
-    enc.append(*i);
-    pos+=i->length()+1;
-    chopOff(chopped);                   // www.powerdns.com -> powerdns.com -> com 
   }
-  enc.append(1,(char)0);
+  d_record.push_back(0);
 
  out:;
-
-  const uint8_t* ptr=reinterpret_cast<const uint8_t*>(enc.c_str());
-  d_record.insert(d_record.end(), ptr, ptr+enc.size());
 }
 
 void DNSPacketWriter::xfrBlob(const string& blob)
@@ -163,6 +231,12 @@ void DNSPacketWriter::xfrBlob(const string& blob)
   d_record.insert(d_record.end(), ptr, ptr+blob.size());
 }
 
+void DNSPacketWriter::xfrHexBlob(const string& blob)
+{
+  xfrBlob(blob);
+}
+
+
 void DNSPacketWriter::getRecords(string& records)
 {
   records.assign(d_content.begin() + d_sor, d_content.end());
