@@ -3,9 +3,9 @@
     Copyright (C) 2002  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    it under the terms of the GNU General Public License version 2
+    as published by the Free Software Foundation
+    
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "utility.hh"
 #include "dns.hh"
@@ -29,7 +29,7 @@
 bool QType::uninit=true;
 vector<QType::namenum> QType::names;
 
-void QType::insert(char *p, int n)
+void QType::insert(const char *p, int n)
 {
   names.push_back(make_pair(string(p),n));
 }
@@ -44,11 +44,13 @@ QType::QType()
       insert("NS",2);
       insert("CNAME",5);
       insert("SOA",6);
+      insert("MR",9);
       insert("PTR",12);
       insert("HINFO",13);
       insert("MX",15);
       insert("TXT",16);
       insert("RP",17);
+      insert("AFSDB", 18);
       insert("SIG",24);
       insert("KEY",25);
       insert("AAAA",28);
@@ -56,6 +58,7 @@ QType::QType()
       insert("SRV",33);
       insert("A6",38);
       insert("NAPTR",35);
+      insert("SPF",99);
       insert("AXFR",252);
       insert("ANY",255);
       insert("URL",256);
